//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2006 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
//////////////////////////////////////////////////////////////////////////////
//   ____  ____
//  /   /\/   /
// /___/  \  /   Vendor:        Xilinx
// \   \   \/    Version:       1.0.0
//  \   \        Filename:      picoCrc.v
//  /   /        Date Created:  October 18, 2007
// /___/   /\    Last Modified: October 18, 2007
// \   \  /  \
//  \___\/\___\
//
// Devices:   Spartan-3 Generation FPGA
// Purpose:   Failsafe multiboot  
// Contact:   jimwe@xilinx.com
// Reference: None
//
// Revision History:
//   Rev 1.0.0 - (jimwe) First created October 18, 2007.
//
//////////////////////////////////////////////////////////////////////////////
//
// LIMITED WARRANTY AND DISCLAIMER. These designs are provided to you "as is".
// Xilinx and its licensors make and you receive no warranties or conditions,
// express, implied, statutory or otherwise, and Xilinx specifically disclaims
// any implied warranties of merchantability, non-infringement, or fitness for
// a particular purpose. Xilinx does not warrant that the functions contained
// in these designs will meet your requirements, or that the operation of
// these designs will be uninterrupted or error free, or that defects in the
// designs will be corrected. Furthermore, Xilinx does not warrant or make any
// representations regarding use or the results of the use of the designs in
// terms of correctness, accuracy, reliability, or otherwise.
//
// LIMITATION OF LIABILITY. In no event will Xilinx or its licensors be liable
// for any loss of data, lost profits, cost or procurement of substitute goods
// or services, or for any special, incidental, consequential, or indirect
// damages arising from the use or operation of the designs or accompanying
// documentation, however caused and on any theory of liability. This
// limitation will apply even if Xilinx has been advised of the possibility
// of such damage. This limitation shall apply not-withstanding the failure
// of the essential purpose of any limited remedies herein.
//
//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2006 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
//////////////////////////////////////////////////////////////////////////////
//
//
///////////////////////////////////////////////////////////////////////
// File:  CRC16_D8.v                             
// Date:  Tue Nov 20 16:41:00 2007                                                      
//                                                                     
// Copyright (C) 1999-2003 Easics NV.                 
// This source file may be used and distributed without restriction    
// provided that this copyright statement is not removed from the file 
// and that any derivative work contains the original copyright notice
// and the associated disclaimer.
//
// THIS SOURCE FILE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS
// OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// Purpose: Verilog module containing a synthesizable CRC function
//   * polynomial: (0 5 12 16)
//   * data width: 8
//                                                                     
// Info: tools@easics.be
//       http://www.easics.com                                  
///////////////////////////////////////////////////////////////////////
  
`timescale 1 ns / 1 ps

module picoCrc
  (
  input   wire        clk,

  input   wire        resetCrc,
  input   wire        useData,
  input   wire [7:0]  Din,
  output  wire [15:0] Cout );

  reg  [15:0] C;
  wire [15:0] NewCRC;
  wire [15:0] D;

  always @(posedge clk)
  begin
    if (resetCrc)
      C <= 16'hFFFF;
    else if (useData)
      C <= NewCRC;
  end

  //assign D = {Din[0], Din[1], Din[2], Din[3],
  //            Din[4], Din[5], Din[6], Din[7]};
  assign D = Din;
  assign Cout = C;

  assign NewCRC[0] = D[4] ^ D[0] ^ C[8] ^ C[12];
  assign NewCRC[1] = D[5] ^ D[1] ^ C[9] ^ C[13];
  assign NewCRC[2] = D[6] ^ D[2] ^ C[10] ^ C[14];
  assign NewCRC[3] = D[7] ^ D[3] ^ C[11] ^ C[15];
  assign NewCRC[4] = D[4] ^ C[12];
  assign NewCRC[5] = D[5] ^ D[4] ^ D[0] ^ C[8] ^ C[12] ^ C[13];
  assign NewCRC[6] = D[6] ^ D[5] ^ D[1] ^ C[9] ^ C[13] ^ C[14];
  assign NewCRC[7] = D[7] ^ D[6] ^ D[2] ^ C[10] ^ C[14] ^ C[15];
  assign NewCRC[8] = D[7] ^ D[3] ^ C[0] ^ C[11] ^ C[15];
  assign NewCRC[9] = D[4] ^ C[1] ^ C[12];
  assign NewCRC[10] = D[5] ^ C[2] ^ C[13];
  assign NewCRC[11] = D[6] ^ C[3] ^ C[14];
  assign NewCRC[12] = D[7] ^ D[4] ^ D[0] ^ C[4] ^ C[8] ^ C[12] ^ C[15];
  assign NewCRC[13] = D[5] ^ D[1] ^ C[5] ^ C[9] ^ C[13];
  assign NewCRC[14] = D[6] ^ D[2] ^ C[6] ^ C[10] ^ C[14];
  assign NewCRC[15] = D[7] ^ D[3] ^ C[7] ^ C[11] ^ C[15];

endmodule
