//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2006 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
//////////////////////////////////////////////////////////////////////////////
//   ____  ____
//  /   /\/   /
// /___/  \  /   Vendor:        Xilinx
// \   \   \/    Version:       1.0.0
//  \   \        Filename:      picojtag.v
//  /   /        Date Created:  October 18, 2007
// /___/   /\    Last Modified: October 18, 2007
// \   \  /  \
//  \___\/\___\
//
// Devices:   Spartan-3 Generation FPGA
// Purpose:   Failsafe multiboot  
// Contact:   jimwe@xilinx.com
// Reference: None
//
// Revision History:
//   Rev 1.0.0 - (jimwe) First created October 18, 2007.
//
//////////////////////////////////////////////////////////////////////////////
//
// LIMITED WARRANTY AND DISCLAIMER. These designs are provided to you "as is".
// Xilinx and its licensors make and you receive no warranties or conditions,
// express, implied, statutory or otherwise, and Xilinx specifically disclaims
// any implied warranties of merchantability, non-infringement, or fitness for
// a particular purpose. Xilinx does not warrant that the functions contained
// in these designs will meet your requirements, or that the operation of
// these designs will be uninterrupted or error free, or that defects in the
// designs will be corrected. Furthermore, Xilinx does not warrant or make any
// representations regarding use or the results of the use of the designs in
// terms of correctness, accuracy, reliability, or otherwise.
//
// LIMITATION OF LIABILITY. In no event will Xilinx or its licensors be liable
// for any loss of data, lost profits, cost or procurement of substitute goods
// or services, or for any special, incidental, consequential, or indirect
// damages arising from the use or operation of the designs or accompanying
// documentation, however caused and on any theory of liability. This
// limitation will apply even if Xilinx has been advised of the possibility
// of such damage. This limitation shall apply not-withstanding the failure
// of the essential purpose of any limited remedies herein.
//
//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2006 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
//////////////////////////////////////////////////////////////////////////////

`timescale 1 ns / 1 ps

module picoJtag
  (
  input   wire        clk,

  input   wire        reset_buffer,
  input   wire        read_buffer,
  input   wire        write_buffer,
  input   wire [7:0]  data_in,
  output  wire        tbuffer_data_present,
  output  wire        tbuffer_half_full,
  output  wire        tbuffer_full,
  output  wire [7:0]  data_out,
  output  wire        rbuffer_data_present,
  output  wire        rbuffer_half_full,
  output  wire        rbuffer_full );
  
  reg  [12:0] jshift_reg;
  reg         jfifo_read;
  reg         jfifo_write;
  reg         jfifo_read_complete;
  reg         jfifo_write_complete;
  reg         update_ff;
  wire [7:0]  tbuffer_dout;

  // Instantiate the boundary scan module
  BSCAN_SPARTAN3 s3_bscan (
    .TDO1(tdo),
    .TDO2(1'b0),
    .UPDATE(update),
    .SHIFT(shift),
    .RESET(),
    .TDI(tdi),
    .SEL1(juse),
    .DRCK1(tck),
    .SEL2(),
    .DRCK2(),
    .CAPTURE(capture));
 
  // Instantiate bufg for the boundary scan module
  BUFG jtag_clock (.I(tck),.O(tck_g));

  // The jtag shift register
    
  assign tdo = jshift_reg[0];

  always @(posedge tck_g)
  begin
    if (capture)
      jshift_reg <= {tbuffer_data_present, tbuffer_full,
                     rbuffer_data_present, rbuffer_full,
                     tbuffer_dout};
    else
      jshift_reg <= {tdi,jshift_reg[12:1]};
  end

  // Resynchronize from jtag clk domain to clk domain
  always @(posedge clk)
    update_ff <= update;

  // Read strobe edge detection
  always @(posedge clk)
  begin
    if (update_ff)
      begin
        jfifo_read <= !jfifo_read_complete & jshift_reg[8];
        jfifo_read_complete <= 1;
      end
    else
      begin
        jfifo_read <= 0;
        jfifo_read_complete <= 0;
      end
  end

  // Write strobe edge detection
  always @(posedge clk)
  begin
    if (update_ff)
      begin
        jfifo_write <= !jfifo_write_complete & jshift_reg[9];
        jfifo_write_complete <= 1;
      end
    else
      begin
        jfifo_write <= 0;
        jfifo_write_complete <= 0;
      end
  end

 // Data fifos
  bbfifo_16x8 rBuffer
  (       .data_in(jshift_reg[7:0]),
          .data_out(data_out),
          .reset(reset_buffer),
          .write(jfifo_write),
          .read(read_buffer),
          .full(rbuffer_full),
          .half_full(rbuffer_half_full),
          .data_present(rbuffer_data_present),
          .clk(clk));

 bbfifo_16x8 tBuffer
  (       .data_in(data_in),
          .data_out(tbuffer_dout),
          .reset(reset_buffer),
          .write(write_buffer),
          .read(jfifo_read),
          .full(tbuffer_full),
          .half_full(tbuffer_half_full),
          .data_present(tbuffer_data_present),
          .clk(clk));

endmodule
