//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2006 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
//////////////////////////////////////////////////////////////////////////////
//   ____  ____
//  /   /\/   /
// /___/  \  /   Vendor:        Xilinx
// \   \   \/    Version:       2.0.0
//  \   \        Filename:      spi.v
//  /   /        Date Created:  October 18, 2007
// /___/   /\    Last Modified: June 10, 2009
// \   \  /  \
//  \___\/\___\
//
// Devices:   Spartan-3 Generation FPGA
// Purpose:   Failsafe multiboot  
// Contact:   jimwe@xilinx.com
// Reference: None
//
// Revision History:
//   Rev 1.0.0 - (jimwe) First created October 18, 2007.
//   Rev 1.1.0 - (jimwe) Removed the unused FPGA_REVISION port
//
//////////////////////////////////////////////////////////////////////////////
//
// LIMITED WARRANTY AND DISCLAIMER. These designs are provided to you "as is".
// Xilinx and its licensors make and you receive no warranties or conditions,
// express, implied, statutory or otherwise, and Xilinx specifically disclaims
// any implied warranties of merchantability, non-infringement, or fitness for
// a particular purpose. Xilinx does not warrant that the functions contained
// in these designs will meet your requirements, or that the operation of
// these designs will be uninterrupted or error free, or that defects in the
// designs will be corrected. Furthermore, Xilinx does not warrant or make any
// representations regarding use or the results of the use of the designs in
// terms of correctness, accuracy, reliability, or otherwise.
//
// LIMITATION OF LIABILITY. In no event will Xilinx or its licensors be liable
// for any loss of data, lost profits, cost or procurement of substitute goods
// or services, or for any special, incidental, consequential, or indirect
// damages arising from the use or operation of the designs or accompanying
// documentation, however caused and on any theory of liability. This
// limitation will apply even if Xilinx has been advised of the possibility
// of such damage. This limitation shall apply not-withstanding the failure
// of the essential purpose of any limited remedies herein.
//
//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2006 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
//////////////////////////////////////////////////////////////////////////////

`timescale 1 ns / 1 ps

module spi
  (
  input   wire        CLK,
  input   wire        RESET,
  
  input   wire [2:0] PICO_ADDR,
  input   wire [7:0] PICO_DATAIN,
  output  reg  [7:0] PICO_DATAOUT,
  input   wire       PICO_WE,
  input   wire       PICO_RD,

  // Connections to the spi device
  input  wire        miso,
  output wire        mosi,
  output reg         sck,
  output reg         ss_b = 1

  );

  reg  [3:0] bit_count = 4'h0;
  reg  [7:0] shift_out_reg;
  reg  [7:0] shift_in_reg;
  reg        busy;

  // Chip select
  always @(posedge CLK)
  begin
    if (RESET)
        ss_b <= 1'b1;
    else if (PICO_WE & (PICO_ADDR == 3'h0))
      begin
        ss_b <= PICO_DATAIN[0];
      end 
  end

  // Register read muxes
  always @(*)
  begin
    case(PICO_ADDR)
      3'h0 : PICO_DATAOUT = { 6'h0, busy, ss_b };
      3'h1 : PICO_DATAOUT = shift_in_reg;             
      default: PICO_DATAOUT = 8'h0;
    endcase
  end

  // Clock register
  always @(posedge CLK)
  begin
    if (RESET)
      sck <= 1'b1;
    else if (bit_count != 4'h0)
      sck <= !sck;
  end

  // Data transfer 

  assign mosi = shift_out_reg[7];

  always @(posedge CLK)
  begin
    if (PICO_WE & (PICO_ADDR == 3'h1))
      begin
        shift_out_reg <= PICO_DATAIN;
        bit_count <= 4'h8;
        busy      <= 1'b1;
      end
    else if (bit_count != 4'h0)
      begin
        if (sck == 1'b0)
          begin
            bit_count <= bit_count - 1'b1;
            shift_in_reg <= {shift_in_reg[6:0], miso};
            if (bit_count == 1'b1)
              busy <= 1'b0;
          end
        else if (bit_count[3] == 1'b0)
          begin
            shift_out_reg <= {shift_out_reg[6:0], 1'b0};
          end
      end
  end

endmodule
