------------------------------------------------------------------------------
-- Copyright (c) 2008 Xilinx, Inc.
-- This design is confidential and proprietary of Xilinx, All Rights Reserved.
------------------------------------------------------------------------------
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor:        Xilinx
-- \   \   \/    Version:       1.0.0
--  \   \        Filename:      picojtag.vhd
--  /   /        Date Created:  April 22, 2008
-- /___/   /\    Last Modified: April 22, 2008
-- \   \  /  \
--  \___\/\___\
--
-- Devices:   Spartan-3 Generation FPGA
-- Purpose:   Failsafe multiboot  
-- Contact:   jimwe@xilinx.com
-- Reference: None
--
-- Revision History:
--   Rev 1.0.0 - (jimwe) First created April 22, 2008.
--
------------------------------------------------------------------------------
--
-- LIMITED WARRANTY AND DISCLAIMER. These designs are provided to you "as is".
-- Xilinx and its licensors make and you receive no warranties or conditions,
-- express, implied, statutory or otherwise, and Xilinx specifically disclaims
-- any implied warranties of merchantability, non-infringement, or fitness for
-- a particular purpose. Xilinx does not warrant that the functions contained
-- in these designs will meet your requirements, or that the operation of
-- these designs will be uninterrupted or error free, or that defects in the
-- designs will be corrected. Furthermore, Xilinx does not warrant or make any
-- representations regarding use or the results of the use of the designs in
-- terms of correctness, accuracy, reliability, or otherwise.
--
-- LIMITATION OF LIABILITY. In no event will Xilinx or its licensors be liable
-- for any loss of data, lost profits, cost or procurement of substitute goods
-- or services, or for any special, incidental, consequential, or indirect
-- damages arising from the use or operation of the designs or accompanying
-- documentation, however caused and on any theory of liability. This
-- limitation will apply even if Xilinx has been advised of the possibility
-- of such damage. This limitation shall apply not-withstanding the failure
-- of the essential purpose of any limited remedies herein.
--
------------------------------------------------------------------------------
-- Copyright (c) 2008 Xilinx, Inc.
-- This design is confidential and proprietary of Xilinx, All Rights Reserved.
------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity PICOREGS is
  port (
    CLK           : in  std_logic;
    APP_ADDR      : in  std_logic;
    APP_DATAIN    : in  std_logic_vector(7 downto 0);
    APP_DATAOUT   : out std_logic_vector(7 downto 0);
    APP_RD        : in  std_logic;
    APP_WE        : in  std_logic;
    RESET_BUFFER  : in  std_logic;
    READ_BUFFER   : in  std_logic;
    WRITE_BUFFER  : in  std_logic;
    DATA_IN       : in  std_logic_vector(7 downto 0);
    DATA_OUT      : out std_logic_vector(7 downto 0);
    TBUFFER_DATA_PRESENT : out std_logic;
    TBUFFER_HALF_FULL    : out std_logic;
    TBUFFER_FULL         : out std_logic;
    RBUFFER_DATA_PRESENT : out std_logic;
    RBUFFER_HALF_FULL    : out std_logic;
    RBUFFER_FULL         : out std_logic 
  );
end PICOREGS;

architecture RTL of PICOREGS is

  component bbfifo_16x8 
    port (
      DATA_IN      : in  std_logic_vector(7 downto 0);
      DATA_OUT     : out std_logic_vector(7 downto 0);
      RESET        : in  std_logic;
      WRITE        : in  std_logic;
      READ         : in  std_logic;
      FULL         : out std_logic;
      HALF_FULL    : out std_logic;
      DATA_PRESENT : out std_logic;
      CLK          : in  std_logic);
  end component;
    

  signal tbuffer_dout : std_logic_vector(7 downto 0);
  signal tbuffer_data_present_i : std_logic;
  signal tbuffer_half_full_i    : std_logic;
  signal tbuffer_full_i         : std_logic;
  signal rbuffer_data_present_i : std_logic;
  signal rbuffer_half_full_i    : std_logic;
  signal rbuffer_full_i         : std_logic;

begin

  -- Assign outputs
  TBUFFER_DATA_PRESENT <= tbuffer_data_present_i;
  TBUFFER_HALF_FULL    <= tbuffer_half_full_i;
  TBUFFER_FULL         <= tbuffer_full_i;
  RBUFFER_DATA_PRESENT <= rbuffer_data_present_i;
  RBUFFER_HALF_FULL    <= rbuffer_half_full_i;
  RBUFFER_FULL         <= rbuffer_full_i;

  APP_DATAOUT <= tbuffer_data_present_i & "00" & tbuffer_full_i &
                 rbuffer_full_i         & "00" & rbuffer_data_present_i
                 when app_addr = '1' else
                 tbuffer_dout;

  -- Data fifos
  rBuffer : bbfifo_16x8 
    port map (
      DATA_IN      => APP_DATAIN,
      DATA_OUT     => DATA_OUT,
      RESET        => RESET_BUFFER,
      WRITE        => APP_WE,
      READ         => READ_BUFFER,
      FULL         => rbuffer_full_i,
      HALF_FULL    => rbuffer_half_full_i,
      DATA_PRESENT => rbuffer_data_present_i,
      CLK          => CLK );

  tBuffer : bbfifo_16x8 
    port map (
      DATA_IN      => DATA_IN,
      DATA_OUT     => TBUFFER_DOUT,
      RESET        => RESET_BUFFER,
      WRITE        => WRITE_BUFFER,
      READ         => APP_RD,
      FULL         => tbuffer_full_i,
      HALF_FULL    => tbuffer_half_full_i,
      DATA_PRESENT => tbuffer_data_present_i,
      CLK          => CLK );

end RTL;
