------------------------------------------------------------------------------
-- Copyright (c) 2008 Xilinx, Inc.
-- This design is confidential and proprietary of Xilinx, All Rights Reserved.
------------------------------------------------------------------------------
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor:        Xilinx
-- \   \   \/    Version:       1.0.0
--  \   \        Filename:      synchro.vhd
--  /   /        Date Created:  April 22, 2008
-- /___/   /\    Last Modified: April 22, 2008
-- \   \  /  \
--  \___\/\___\
--
-- Devices:   Spartan-3 Generation FPGA
-- Purpose:   Signal synchronizer, for async inputs
-- Contact:   crabill@xilinx.com
-- Reference: None
--
-- Revision History:
--   Rev 1.0.0 - (crabill) First created October 1, 2006.
--
------------------------------------------------------------------------------
--
-- LIMITED WARRANTY AND DISCLAIMER. These designs are provided to you "as is".
-- Xilinx and its licensors make and you receive no warranties or conditions,
-- express, implied, statutory or otherwise, and Xilinx specifically disclaims
-- any implied warranties of merchantability, non-infringement, or fitness for
-- a particular purpose. Xilinx does not warrant that the functions contained
-- in these designs will meet your requirements, or that the operation of
-- these designs will be uninterrupted or error free, or that defects in the
-- designs will be corrected. Furthermore, Xilinx does not warrant or make any
-- representations regarding use or the results of the use of the designs in
-- terms of correctness, accuracy, reliability, or otherwise.
--
-- LIMITATION OF LIABILITY. In no event will Xilinx or its licensors be liable
-- for any loss of data, lost profits, cost or procurement of substitute goods
-- or services, or for any special, incidental, consequential, or indirect
-- damages arising from the use or operation of the designs or accompanying
-- documentation, however caused and on any theory of liability. This
-- limitation will apply even if Xilinx has been advised of the possibility
-- of such damage. This limitation shall apply not-withstanding the failure
-- of the essential purpose of any limited remedies herein.
--
------------------------------------------------------------------------------
-- Copyright (c) 2008 Xilinx, Inc.
-- This design is confidential and proprietary of Xilinx, All Rights Reserved.
------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;

library UNISIM;
use UNISIM.VCOMPONENTS.all;

entity SYNCHRO is
  generic (
    INITIALIZE : string := "LOGIC0"
  );
  port (
    ASYNC : in  std_logic;
    CLK   : in  std_logic;
    SYNC  : out std_logic
  );
end SYNCHRO;

architecture RTL of SYNCHRO is

  signal temp : std_logic;

  attribute ASYNC_REG : string;
  attribute HU_SET    : string;
  attribute RLOC      : string;
  --attribute ASYNC_REG of fda:label is "TRUE";
  --attribute ASYNC_REG of fdb:label is "TRUE";
  --attribute HU_SET    of fda:label is "SYNC";
  --attribute HU_SET    of fdb:label is "SYNC";
  --attribute RLOC      of fda:label is "X0Y0";
  --attribute RLOC      of fdb:label is "X0Y0";

begin

  ----------------------------------------------------------------------
  -- Synchronizer.                                                    --
  ----------------------------------------------------------------------


  use_fdp : if (INITIALIZE = "LOGIC1") generate
    attribute ASYNC_REG of fda:label is "TRUE";
    attribute ASYNC_REG of fdb:label is "TRUE";
    attribute HU_SET    of fda:label is "SYNC";
    attribute HU_SET    of fdb:label is "SYNC";
    attribute RLOC      of fda:label is "X0Y0";
    attribute RLOC      of fdb:label is "X0Y0";
  begin
      fda : FDP
        port map (
          Q   => temp,
          D   => async,
          C   => clk,
          PRE => '0' );
      fdb : FDP
        port map (
          Q   => sync,
          D   => temp,
          C   => clk,
          PRE => '0' );
  end generate;

  use_fdc : if (INITIALIZE = "LOGIC0") generate
    attribute ASYNC_REG of fda:label is "TRUE";
    attribute ASYNC_REG of fdb:label is "TRUE";
    attribute HU_SET    of fda:label is "SYNC";
    attribute HU_SET    of fdb:label is "SYNC";
    attribute RLOC      of fda:label is "X0Y0";
    attribute RLOC      of fdb:label is "X0Y0";
  begin
      fda : FDC
        port map (
          Q   => temp,
          D   => async,
          C   => clk,
          CLR => '0' );
      fdb : FDC
        port map (
          Q   => sync,
          D   => temp,
          C   => clk,
          CLR => '0' );
  end generate;

end RTL;
