------------------------------------------------------------------------------
-- Copyright (c) 2008 Xilinx, Inc.
-- This design is confidential and proprietary of Xilinx, All Rights Reserved.
------------------------------------------------------------------------------
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor:        Xilinx
-- \   \   \/    Version:       2.0.0
--  \   \        Filename:      spi.vhd
--  /   /        Date Created:  April 22, 2008
-- /___/   /\    Last Modified: April 22, 2008
-- \   \  /  \
--  \___\/\___\
--
-- Devices:   Spartan-3 Generation FPGA
-- Purpose:   Failsafe multiboot  
-- Contact:   jimwe@xilinx.com
-- Reference: None
--
-- Revision History:
--   Rev 1.0.0 - (jimwe) First created April 22, 2008.
--
------------------------------------------------------------------------------
--
-- LIMITED WARRANTY AND DISCLAIMER. These designs are provided to you "as is".
-- Xilinx and its licensors make and you receive no warranties or conditions,
-- express, implied, statutory or otherwise, and Xilinx specifically disclaims
-- any implied warranties of merchantability, non-infringement, or fitness for
-- a particular purpose. Xilinx does not warrant that the functions contained
-- in these designs will meet your requirements, or that the operation of
-- these designs will be uninterrupted or error free, or that defects in the
-- designs will be corrected. Furthermore, Xilinx does not warrant or make any
-- representations regarding use or the results of the use of the designs in
-- terms of correctness, accuracy, reliability, or otherwise.
--
-- LIMITATION OF LIABILITY. In no event will Xilinx or its licensors be liable
-- for any loss of data, lost profits, cost or procurement of substitute goods
-- or services, or for any special, incidental, consequential, or indirect
-- damages arising from the use or operation of the designs or accompanying
-- documentation, however caused and on any theory of liability. This
-- limitation will apply even if Xilinx has been advised of the possibility
-- of such damage. This limitation shall apply not-withstanding the failure
-- of the essential purpose of any limited remedies herein.
--
------------------------------------------------------------------------------
-- Copyright (c) 2008 Xilinx, Inc.
-- This design is confidential and proprietary of Xilinx, All Rights Reserved.
------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity SPI is
  port (
    CLK           : in  std_logic;
    RESET         : in  std_logic;
    PICO_ADDR     : in  std_logic_vector(2 downto 0);
    PICO_DATAIN   : in  std_logic_vector(7 downto 0);
    PICO_DATAOUT  : out std_logic_vector(7 downto 0);
    PICO_WE       : in  std_logic;
    PICO_RD       : in  std_logic;
    MISO          : in  std_logic;
    MOSI          : out std_logic;
    SCK           : out std_logic;
    SS_B          : out std_logic
  );
end SPI;

architecture RTL of SPI is 

  signal bit_count     : std_logic_vector(3 downto 0) := "0000";
  signal shift_out_reg : std_logic_vector(7 downto 0);
  signal shift_in_reg  : std_logic_vector(7 downto 0);
  signal busy          : std_logic;
  signal ss_b_i        : std_logic := '1';
  signal sck_i         : std_logic;

begin

  -- Output assignments
  SS_B <= ss_b_i;
  SCK  <= sck_i;

  -- SPI chip select generation
  cs_process : process(CLK)
  begin
    if rising_edge(CLK) then
      if (RESET = '1') then
        ss_b_i <= '1';
      elsif (PICO_WE = '1' and (PICO_ADDR = "000")) then      
        ss_b_i <= PICO_DATAIN(0);
      end if;
    end if;
  end process cs_process;

  -- Read muxes
  read_mux_process : process(busy, ss_b_i, shift_in_reg, PICO_ADDR)
  begin
    case PICO_ADDR is
      when "000"  => PICO_DATAOUT <= "000000" & busy & ss_b_i;
      when "001"  => PICO_DATAOUT <= shift_in_reg;
      when others => PICO_DATAOUT <= (others => '0');
    end case;
  end process read_mux_process;

  -- Clock generation
  sck_process : process(CLK)
  begin
    if rising_edge(CLK) then
      if (RESET = '1') then
         sck_i <= '1';
      elsif (bit_count /= "0000") then
         sck_i <= not sck_i;
      end if;
    end if;
  end process sck_process;


  -- Shifting logic
  MOSI <= shift_out_reg(7);

  shift_process : process(CLK)
  begin
    if rising_edge(CLK) then
      if ((PICO_WE = '1') and (PICO_ADDR = "001")) then
         shift_out_reg <= PICO_DATAIN;
         bit_count <= "1000";
         busy      <= '1';
      elsif (bit_count /= "0000") then
         if (sck_i = '0') then
           bit_count <= bit_count - "0001";
           shift_in_reg <= shift_in_reg(6 downto 0) & MISO;
           if (bit_count = "0001") then
             busy <= '0';
           end if;
         elsif (bit_count(3) = '0') then
           shift_out_reg <= shift_out_reg(6 downto 0) & '0';
        end if;
      end if;
    end if;
  end process shift_process;

end RTL;
